@@ -1,3 +1,5 @@
+import { Observable, Subject } from "rxjs";
+import { filter, first, takeUntil } from "rxjs/operators";
 import { Injectable } from "@angular/core";
 import {
     extend,
@@ -8,7 +10,6 @@ import {
     TupleOfflineStorageNameService,
     TupleOfflineStorageService,
     TupleSelector,
-    TupleStorageBatchSaveArguments,
     TupleStorageFactoryService,
     VortexService,
     VortexStatusService,
@@ -19,17 +20,17 @@ import {
     docDbFilt,
     docDbTuplePrefix,
 } from "../PluginNames";
-
-import { Observable, Subject } from "rxjs";
 import { EncodedDocumentChunkTuple } from "./EncodedDocumentChunkTuple";
 import { DocumentUpdateDateTuple } from "./DocumentUpdateDateTuple";
 import { DocumentTuple } from "../../DocumentTuple";
 import { DocDbTupleService } from "../DocDbTupleService";
 import { DocDbDocumentTypeTuple } from "../../DocDbDocumentTypeTuple";
 import { PrivateDocumentLoaderStatusTuple } from "./PrivateDocumentLoaderStatusTuple";
-
-import { OfflineConfigTuple } from "../tuples/OfflineConfigTuple";
 import { DocDbModelSetTuple } from "../../DocDbModelSetTuple";
+import {
+    DeviceOfflineCacheControllerService,
+    OfflineCacheStatusTuple,
+} from "@peek/peek_core_device";
 
 // ----------------------------------------------------------------------------
 
@@ -139,36 +140,19 @@ export class PrivateDocumentLoaderService extends NgLifeCycleEvents {
     private modelSetByIds: { [id: number]: DocDbModelSetTuple } = {};
     private _hasModelSetLoaded = false;
 
-    private offlineConfig: OfflineConfigTuple = new OfflineConfigTuple();
-
     constructor(
         private vortexService: VortexService,
         private vortexStatusService: VortexStatusService,
         storageFactory: TupleStorageFactoryService,
-        private tupleService: DocDbTupleService
+        private tupleService: DocDbTupleService,
+        private deviceCacheControllerService: DeviceOfflineCacheControllerService
     ) {
         super();
 
-        this.tupleService.offlineObserver
-            .subscribeToTupleSelector(
-                new TupleSelector(OfflineConfigTuple.tupleName, {}),
-                false,
-                false,
-                true
-            )
-            .takeUntil(this.onDestroyEvent)
-            .filter((v) => v.length != 0)
-            .subscribe((tuples: OfflineConfigTuple[]) => {
-                this.offlineConfig = tuples[0];
-                if (this.offlineConfig.cacheChunksForOffline)
-                    this.initialLoad();
-                this._notifyStatus();
-            });
-
         let objTypeTs = new TupleSelector(DocDbDocumentTypeTuple.tupleName, {});
         this.tupleService.offlineObserver
             .subscribeToTupleSelector(objTypeTs)
-            .takeUntil(this.onDestroyEvent)
+            .pipe(takeUntil(this.onDestroyEvent))
             .subscribe((tuples: DocDbDocumentTypeTuple[]) => {
                 this.objectTypesByIds = {};
                 for (let item of tuples) {
@@ -181,7 +165,7 @@ export class PrivateDocumentLoaderService extends NgLifeCycleEvents {
         let modelSetTs = new TupleSelector(DocDbModelSetTuple.tupleName, {});
         this.tupleService.offlineObserver
             .subscribeToTupleSelector(modelSetTs)
-            .takeUntil(this.onDestroyEvent)
+            .pipe(takeUntil(this.onDestroyEvent))
             .subscribe((tuples: DocDbModelSetTuple[]) => {
                 this.modelSetByIds = {};
                 for (let item of tuples) {
@@ -199,10 +183,13 @@ export class PrivateDocumentLoaderService extends NgLifeCycleEvents {
         this.setupVortexSubscriptions();
         this._notifyStatus();
 
-        // Check for updates every so often
-        Observable.interval(this.OFFLINE_CHECK_PERIOD_MS)
-            .takeUntil(this.onDestroyEvent)
-            .subscribe(() => this.askServerForUpdates());
+        this.deviceCacheControllerService.triggerCachingObservable
+            .pipe(takeUntil(this.onDestroyEvent))
+            .pipe(filter((v) => v))
+            .subscribe(() => {
+                this.initialLoad();
+                this._notifyStatus();
+            });
     }
 
     isReady(): boolean {
@@ -236,7 +223,7 @@ export class PrivateDocumentLoaderService extends NgLifeCycleEvents {
 
         // If there is no offline support, or we're online
         if (
-            !this.offlineConfig.cacheChunksForOffline ||
+            !this.deviceCacheControllerService.cachingEnabled ||
             this.vortexStatusService.snapshot.isOnline
         ) {
             let ts = new TupleSelector(DocumentTuple.tupleName, {
@@ -248,8 +235,8 @@ export class PrivateDocumentLoaderService extends NgLifeCycleEvents {
                 .isOnline
                 ? Promise.resolve()
                 : this.vortexStatusService.isOnline
-                      .filter((online) => online)
-                      .first()
+                      .pipe(filter((online) => online))
+                      .pipe(first())
                       .toPromise();
 
             return isOnlinePromise
@@ -268,7 +255,7 @@ export class PrivateDocumentLoaderService extends NgLifeCycleEvents {
             );
 
         return this.isReadyObservable()
-            .first()
+            .pipe(first())
             .toPromise()
             .then(() => this.getDocumentsWhenReady(modelSetKey, keys))
             .then((docs) => this._populateAndIndexObjectTypes(docs));
@@ -285,7 +272,7 @@ export class PrivateDocumentLoaderService extends NgLifeCycleEvents {
 
     private _notifyStatus(): void {
         this._status.cacheForOfflineEnabled =
-            this.offlineConfig.cacheChunksForOffline;
+            this.deviceCacheControllerService.cachingEnabled;
         this._status.initialLoadComplete = this.index.initialLoadComplete;
 
         this._status.loadProgress = Object.keys(
@@ -297,6 +284,15 @@ export class PrivateDocumentLoaderService extends NgLifeCycleEvents {
             ).length;
 
         this._statusSubject.next(this._status);
+
+        const status = new OfflineCacheStatusTuple();
+        status.pluginName = "peek_core_docdb";
+        status.indexName = "Document Index";
+        status.loadingQueueCount = this._status.loadProgress;
+        status.totalLoadedCount = this._status.loadTotal;
+        status.lastCheckDate = new Date();
+        status.initialFullLoadComplete = this._status.initialLoadComplete;
+        this.deviceCacheControllerService.updateCachingStatus(status);
     }
 
     /** Initial load
@@ -331,21 +327,21 @@ export class PrivateDocumentLoaderService extends NgLifeCycleEvents {
                 this,
                 clientDocumentWatchUpdateFromDeviceFilt
             )
-            .takeUntil(this.onDestroyEvent)
+            .pipe(takeUntil(this.onDestroyEvent))
             .subscribe((payloadEnvelope: PayloadEnvelope) => {
                 this.processDocumentsFromServer(payloadEnvelope);
             });
 
         // If the vortex service comes back online, update the watch grids.
         this.vortexStatusService.isOnline
-            .filter((isOnline) => isOnline == true)
-            .takeUntil(this.onDestroyEvent)
+            .pipe(filter((isOnline) => isOnline == true))
+            .pipe(takeUntil(this.onDestroyEvent))
             .subscribe(() => this.askServerForUpdates());
     }
 
     private areWeTalkingToTheServer(): boolean {
         return (
-            this.offlineConfig.cacheChunksForOffline &&
+            this.deviceCacheControllerService.cachingEnabled &&
             this.vortexStatusService.snapshot.isOnline
         );
     }
@@ -407,7 +403,7 @@ export class PrivateDocumentLoaderService extends NgLifeCycleEvents {
 
         for (let key of keysNeedingUpdate) {
             indexChunk.updateDateByChunkKey[key] =
-                this.index.updateDateByChunkKey[key];
+                this.index.updateDateByChunkKey[key] || "";
             count++;
 
             if (count == this.UPDATE_CHUNK_FETCH_SIZE) {
@@ -496,19 +492,26 @@ export class PrivateDocumentLoaderService extends NgLifeCycleEvents {
      * Stores the index bucket in the local db.
      */
     private storeDocumentChunkTuples(
-        tuplesToSave: EncodedDocumentChunkTuple[]
+        encodedDocumentChunkTuples: EncodedDocumentChunkTuple[]
     ): Promise<void> {
-        const Selector = DocumentChunkTupleSelector;
-        const batchStore: TupleStorageBatchSaveArguments[] = [];
+        let retPromise: any;
+        retPromise = this.storage.transaction(true).then((tx) => {
+            let promises = [];
 
-        for (const tuple of tuplesToSave) {
-            batchStore.push({
-                tupleSelector: new Selector(tuple.chunkKey),
-                vortexMsg: tuple.encodedData,
-            });
-        }
+            for (let encodedDocumentChunkTuple of encodedDocumentChunkTuples) {
+                promises.push(
+                    tx.saveTuplesEncoded(
+                        new DocumentChunkTupleSelector(
+                            encodedDocumentChunkTuple.chunkKey
+                        ),
+                        encodedDocumentChunkTuple.encodedData
+                    )
+                );
+            }
 
-        return this.storage.batchSaveTuplesEncoded(batchStore);
+            return Promise.all(promises).then(() => tx.close());
+        });
+        return retPromise;
     }
 
     /** Get Documents When Ready
